/* This file was generated by syncqt. */
#ifndef QT_QTKMSSUPPORT_VERSION_H
#define QT_QTKMSSUPPORT_VERSION_H

#define QTKMSSUPPORT_VERSION_STR "5.15.2"

#define QTKMSSUPPORT_VERSION 0x050F02

#endif // QT_QTKMSSUPPORT_VERSION_H
