/* This file was generated by syncqt. */
#ifndef QT_QTQMLDEBUG_VERSION_H
#define QT_QTQMLDEBUG_VERSION_H

#define QTQMLDEBUG_VERSION_STR "5.15.2"

#define QTQMLDEBUG_VERSION 0x050F02

#endif // QT_QTQMLDEBUG_VERSION_H
