/* This file was generated by syncqt. */
#ifndef QT_QTPLATFORMHEADERS_VERSION_H
#define QT_QTPLATFORMHEADERS_VERSION_H

#define QTPLATFORMHEADERS_VERSION_STR "5.15.2"

#define QTPLATFORMHEADERS_VERSION 0x050F02

#endif // QT_QTPLATFORMHEADERS_VERSION_H
